#ifndef _QWK_H_
#define _QWK_H_

#include <sys/types.h>

#if defined (__linux__)
#include <sys/param.h>
#else
#define MAXPATHLEN 255
typedef unsigned char unchar;
typedef unsigned short ushort;
typedef unsigned long ulong;
#endif

#ifdef __linux__
# define MKDIR(p1,p2) mkdir(p1,p2)
#else
# define MKDIR(p1,p2) mkdir(p1)
#endif

/********************************************************************
    1       1   Message status flag (unsigned character)
                ' ' = public, unread
                '-' = public, read
                '*' = private, read by someone but not by intended
                recipient
                '+' = private, read by official recipient
                '~' = comment to Sysop, unread
                '`' = comment to Sysop, read
                '%' = sender password protected, unread
                '^' = sender password protected, read
                '!' = group password protected, unread
                '#' = group password protected, read
                '$' = group password protected to all
    2       7   Message number (in ASCII)
    9       8   Date (mm-dd-yy, in ASCII)
   17       5   Time (24 hour hh:mm, in ASCII)
   22      25   To (uppercase, left justified)
   47      25   From (uppercase, left justified)
   72      25   Subject of message (mixed case)
   97      12   Password (space filled)
  109       8   Reference message number (in ASCII)
  117       6   Number of 128-bytes blocks in message (including the
                header, in ASCII; the lowest value should be 2, header
                plus one block message; this number may not be left
                flushed within the field)
  123       1   Flag (ASCII 225 means message is active; ASCII 226
                means this message is to be killed)
  124       2   Conference number (unsigned word)
  126       2   Logical message number in the current packet; i.e.
                this number will be 1 for the first message, 2 for the
                second, and so on. (unsigned word)
  128       1   Indicates whether the message has a network tag-line
                or not.  A value of '*' indicates that a network tag-
                line is present; a value of ' ' (space) indicates
                there isn't one.  Messages sent to readers (non-net-
                status) generally leave this as a space.  Only network
                softwares need this information.
************************************************************************/

#if defined (__linux__) || defined(AMIGA)
# define PATHSEP '/'
#else
# define PATHSEP '\\'
#endif

typedef struct
{
    char msflag;
    char mnumber[7];
    char mdate[8];
    char mtime[5];
    char mto[25];
    char mfrom[25];
    char msubj[25];
    char mpass[12];
    char mref[8];
    char mblocks[6];
    char mflag;
    unchar conf[2];   /* This is ushort. It's also on an odd address */
                     /* warning 68000s, RISC machines etc   */
    unchar lmsgno[2]; /* So is this, nihil illegitimes carborundum */
    char mtag;
} QWKREC;

int recmatch (char *, char *);
ulong msb2ul (unchar *);

#ifdef __linux__
int strnicmp (const char *, const char *, int);
int stricmp (const char *, const char *);
#endif

#if defined (__NT__) || defined (__OS2__) || defined (__DOS__)
char * stpcpy(char *, const char *);
#endif

#ifdef _DCC
void putenv(char *);
#define CWDSIZE 255
#else
#define CWDSIZE 0
#endif

void * xmalloc (size_t);
void deldir(char *);


#endif
