 /*
        Qwk2Fido - Unpack any QWK reply bundles in inbound,
        Process received reply packets from an offline reader.
 */
 
#include <stdio.h>
#include <stdlib.h>
#include <qdos.h>
#include <qptr.h>
#include <time.h>
#include <unistd.h>
#include <string.h>
#include <ctype.h>
#include <csrvthg.h>
#include <database.h>

#define unixtoql 283996800              /* time conversion, 1970 to 1961 */

typedef struct
{
    char status;
    char msgnum[7];
    char date[8];
    char time[5];
    char msgto[25];
    char msgfrom[25];
    char subject[25];
    char password[12];
    char reply[8];
    char numblocks[6];
    char active;
    char confl;
    char confh;
    char numl;
    char numh;
    char tagline;
}  QWKHEADER;

#define BMAX 42
char _prog_name[] = "Qwk2fido";
char _version[] = " v1.20 ";
char terminate[] = { '\r', '\n', '\r', '\n', '\0' };
long qltime = 0;
long rnum;
long flch = -1;
long dbid = -1;
char exitmsg[82];
char sysdir[82];
char tempdir[82];
char inbound[BMAX];
char store[BMAX];
char mareas[BMAX];
char logfile[BMAX];
char olrdir[BMAX];
char userdir[BMAX];
char logmsg[82];
char address[BMAX];
char areaname[BMAX] = "";
short areanum = 0;
int  line = 0;
long pktfile = -1;
long fchan = -1;
long olrlog;
long replychan;
char *mgroups;
int loglevel;

struct tm *now;
unsigned short myzone = 0,mynet = 0,mynode = 0,mypoint = 0,myfake = 0;

    /*
       This is a table of compressors and their identification IDs.
       It is a sequence of identifying characters to check for.
    */

char zipID[] = { 'P','K', 3, 4, '\0' };       /* ZIP files */
char lzhID[] = { 1, 1,'-','l','h', '\0' };    /* LZH and HAR files */
char arjID[] = { 0x60, -22, '\0'};            /* ARJ files */
char arcID[] = { 26, '\0' };                  /* ARC files */

char    unzip[BMAX], unlzh[BMAX], unarj[BMAX],
        unarc[BMAX], unhar[BMAX], unpcmd[BMAX];

int     unpack    (char * fname);
long    getbuff   (char *s, long n, FILE *iop); /* custom read function */
int     copyfile  (char *s, char *t, char *f);  /* copy file */
int findref(long db, char *rec, struct fld_list fldef[],char *appcn);
int findanum(long db, char *rec, struct fld_list fldef[],char *appcn);
int findaname(long db, char *rec, struct fld_list fldef[],char *appcn);
void intel( long chan, int value);
void sysvar(char *varname, char *ans, int sz);
void log(int level, char *value);
void add_to_pkt(QWKHEADER *hdr);
void open_pkt(void);
void close_pkt(void);


/*---------------------------------------------------------------------------*/
int get_areanum(char *p)
{
        /* get area number, given ptr to area name or number */

int find;
short a;
        if(isdigit(*p))
          return (atoi(p));

        find = fsd_srch(dbid, findaname, p);
        if (find >= 0)
        {
           find = fsd_get(dbid, 1, 2, &a);
           return(a);
        }
        return(0);
}
/*---------------------------------------------------------------------------*/
void add_reply(char *t, int o)
{
char reply[80];
char *ptr;
int l;
          strcpy(reply,t);
          ptr = reply;
          l= strlen(reply);
          ptr += l;
          while(l < 50)
          {
            *ptr = ' ';
            ptr++;
            l++;
          }
          if(o == 1)
            strcpy(ptr," OK\n");
          else if(o == 2)
            strcpy(ptr," No change\n");
          else if(o == 3)
            strcpy(ptr," File not found\n");
          if(o >= 0)
            io_sstrg(replychan,-1,reply,strlen(reply));  /* no reply on fail */
          if(o < 0)
            strcpy(ptr," FAIL\n");
          io_sstrg(olrlog,-1,reply,strlen(reply)); /* but log fails for sysop */
}
/*---------------------------------------------------------------------------*/
int attach(char *name)
{
/*
        Attach the file requested. Return 1 = ok, 3 = not found
*/
char temp[80];
short r;
char *ptr;
long chan,sz;
char *n;
        sprintf(temp,"reqfile %d %s",line,name);
        ptr=Request ("pbox", temp, strlen(temp), &r);
        if (ptr == NULL)
           return(3);
        if(*ptr == '-')
        {
          free(ptr);
          return(3);
        }
        sprintf(temp,"%staglist0",tempdir);
        n = temp;
        n += strlen(temp);
        n--;
        *n += line;
        chan = io_open(temp,0);                 /* open taglist */
        if(chan < 0)
          chan = io_open(temp,2);               /* or new taglist */
        fs_pos(chan,0,2);                       /* goto end of file */
        io_sstrg(chan,-1,ptr,strlen(ptr));
        io_sbyte(chan,-1,10);
        io_close(chan);
        free(ptr);
        sprintf(temp,"setline %d tagn +1",line);
        ptr=Request ("pbox", temp, strlen(temp), &r);
        if (ptr == NULL)
           return(3);
        chan = io_open(ptr,1);                  /* open the requested file */
        sz = fs_pos(chan,0,2);
        sz += 1023;
        sz /= 1024;
        io_close(chan);
        free(ptr);
        sprintf(temp,"setline %d tagk +%d",line,sz);
        ptr=Request ("pbox", temp, strlen(temp), &r);
        if (ptr != NULL) free(ptr);
        return(1);
}
/*---------------------------------------------------------------------------*/
int setopt( char c, char *type)
{
/*
        Type = ON OFF or neither for "once only"
        Return -1 = fail, 1 = ok, 2 = no change
*/
char temp[80];
short r,f;
char l;
char *opt,*ptr;
int ans;
        sprintf(temp,"getline %d oopt",line);
        opt=Request ("pbox", temp, strlen(temp), &r);
        if (opt == NULL) return(-1);
        l = tolower(c);
        r = strlen(opt);
        if(strnicmp("ON",type,2) == 0)  /* add uppercase letter */
        {
          for(f=0; f<r; f++)
            if(opt[f] == l) opt[f] = ' ';       /* remove any lowercase */
          ptr = strchr(opt,c);  /* see if already set */
          if(ptr != NULL)
            ans = 2;
          else
          {
            for(f=0; f<r; f++)
            {
              if(opt[f] == ' ')
              {
                opt[f] = c;
                break;
              }
            }
            if(opt[f] != c)
            {
              opt[r] = c;
              opt[r+1] = '\0';
            }
            ans = 1;
          }
        }
        else if (strnicmp("OFF",type,3) == 0)
        {
          ans = 2;              /* assume no change */
          for(f=0; f<r; f++)
            if(opt[f] == l)
            {
              opt[f] = ' ';     /* remove any lowercase */
              ans = 1;
            }
          for(f=0; f<r; f++)
            if(opt[f] == c)
            {
              opt[f] = ' ';       /* remove any uppercase */
              ans = 1;
            }
        }
        else            /* one shot */
        {
          ans = 2;
          for(f=0; f<r; f++)
            if(opt[f] == c)
            {
              opt[f] = ' ';       /* remove any uppercase */
              ans = 1;
            }
          ptr = strchr(opt,l);  /* see if already set */
          if(ptr == NULL)
          {
            for(f=0; f<r; f++)
            {
              if(opt[f] == ' ')
              {
                opt[f] = l;
                break;
              }
            }
            if(opt[f] != l)
            {
              opt[r] = l;
              opt[r+1] = '\0';
            }
            ans = 1;
          }
        }
        if(ans != 2)
        {
          sprintf(temp,"setline %d oopt %s",line,opt);
          ptr=Request ("pbox", temp, strlen(temp), &r);
          if (ptr != NULL) free(ptr);
        }
        free(opt);
        return(ans);
}
/*---------------------------------------------------------------------------*/
int areafix( short area, char vtype, int val, int mode)
{
/*
        Mode 1 = add, 2 = drop, 3 = reset, 4 = your, 5 = your/all
        Val = zero or message number
        Vtype = + for absolute number, -ve for most recent number
        Return -1 = fail, 1 = ok, 2 = no change

 Adding an area which is already there returns 2 unless val is non-zero or
 val=0 and vtype != ' ', in which case it is treated as a pointer reset.
 Your/YourAll just set the "type" flag, then behave as Add.
*/
char temp[BMAX];
char *ptr;
short unum;
long chn,dbd;
long fch,dbi;
long ich,ibd;
int find;
int ans;
int posn;
int changes = 0;
long ltime;
char c;

        sprintf(temp,"getline %d ltim",line);
        ptr=Request ("pbox", temp, strlen(temp), &unum);
        if (ptr == NULL) return(-1);
        ltime = (short)atoi(ptr);
        free(ptr);
        sprintf(temp,"getline %d mareas",line);
        mgroups=Request ("pbox", temp, strlen(temp), &unum);
        sprintf(temp,"getline %d user",line);
        ptr=Request ("pbox", temp, strlen(temp), &unum);
        if (ptr == NULL) return(-1);
        unum = (short)atoi(ptr);
        free(ptr);
        sprintf(temp,"%s%04xolr_dbs",userdir,unum);
        chn = io_open(temp,0);                          /* open read/write */
        if(chn < 0)
          return(-1);
        dbd = fsd_open(chn);
        if(dbd < 0)
        {
          io_close(chn);
          return(-1);
        }
        changes = 0;
        find = fsd_srch(dbd, findanum, &area);
        if (find < 0)                           /* area not found in user file */
        {
          if(mode == 2)
            ans = 2;                            /* drop an unlinked area */
          else
          {
            ans = 1;
            fsd_put(dbd, 1, 2, &area);          /* create area entry */
            fsd_put(dbd, 2, 4, &ltime);         /* set time to last call */
            fsd_put(dbd, 3, 0, "");             /* clear msgid */
            fsd_put(dbd, 4, 0, "");             /* clear mode  */
            if(mode == 4)
              fsd_put(dbd, 4, 1, "Y");
            if(mode == 5)
              fsd_put(dbd, 4, 2, "YA");
            fsd_appn(dbd);                      /* write to file */
          }
        }
        else                                    /* amend an existing area */
        {
          if(mode == 2)         /* drop the area */
          {
            fsd_del(dbd);
            ans = 1;
          }
          else if(mode == 4)    /* Your */
          {
            fsd_put(dbd, 4, 1, "Y");
            ans = 1;
            changes = 1;
          }
          else if(mode == 5)    /* Your/All */
          {
            fsd_put(dbd, 4, 2, "YA");
            ans = 1;
            changes = 1;
          }
        }
/*
        now process pointer
*/
        if(mode != 2)           /* ignore pointers on deleted areas */
        {
          if((val != 0) || ((val == 0) && (vtype != ' ')))
          {
            posn = val;                 /* if you were given a pointer */
            if(vtype == '-')
              posn = fsd_rcnt(dbd) - val;
            if(posn < 0)
              posn = 0;
          }
          else
          {
            posn = -1;                   /* default if not supplied */
          }
          fsd_put(dbd, 2, 4, &ltime);         /* set time to last call */
          fsd_put(dbd, 3, 0, "");             /* clear msgid */
          find = fsd_srch(dbid, findanum, &area);
          if (find >= 0)              /* found right area in mareas */
          {
            /*
                Check user is allowed access to area. If so, get area
                database name, create index name, position index
                to right message, read msgid and time and store them.
            */
            ans = -1;                 /* assume some problem - pessimist! */
            changes = 0;
            find = fsd_get(dbid, 5, BMAX, temp); /* get group details */
            if (find > 0)               /* it's in a group */
            {
               c = temp[0];                     /* get group letter */
               if(strchr(mgroups,c) == NULL)    /* oops - not allowed */
               {
                 posn = -1;                     /* set "no pointer" flag */
               }
            }
            find = fsd_get(dbid, 4, BMAX, temp); /* area dbs filename */
            if ((find >= 0) && (posn > 0))
            {
              temp[find] = '\0';
              fch = io_open( temp, 1 );     /* open dbs read only */
              if (fch > 0 )
                 dbi = fsd_open(fch);
              if (fch > 0 && dbi > 0 )
              {
                ptr = temp;
                ptr += strlen(temp);
                ptr -= 3;
                (void)strcpy(ptr,"idx");
                ich = io_open( temp, 1 );     /* open index read only */
                if (ich > 0 )
                  ibd = fsd_open(ich);
                if (ich > 0 && ibd > 0 )
                {
                  find = fsd_posa(ibd, posn);
                  find = fsd_get(ibd, 1, BMAX, temp); /* get msgid */
                  if (find >= 0)
                    temp[find] = '\0';
                  else
                    find = 0;
                  fsd_put(dbd, 3, find, temp);      /* store msgid */
                  find = fsd_get(ibd, 2, 4, temp);  /* get "recv" time */
                  fsd_put(dbd, 2, 4, temp);         /* store it */
                  changes++;
                  ans = 1;
                  (void)fsd_close(ibd);
                  (void)io_close(ich);
                }
                (void)fsd_close(dbi);
                (void)io_close(fch);
              }
            }
            if(ans == -1)       /* failed to access the messagebase */
            {                   /* or user denied access to it */
              fsd_del(dbd);
              changes = 0;
            }
          }
          else                /* tried to link to non-existent area */
          {
            fsd_del(dbd);
            ans = -1;
            changes = 0;
          }
        }
        if(changes)
          fsd_updt(dbd);      /* write any changes to user file */
        fsd_close(dbd);
        io_close(chn);
        return(ans);
}
/*---------------------------------------------------------------------------*/
void process_request(QWKHEADER *hdr)
{
int ferr,f;
char lbuff[4096];       /* assume server requests wont exceed this size */
char *ptr,*lptr,*aptr;
char subj[25];
char temp[80];
char c;
int val;
int sz = 0;
short anum;
int ok;
char replymsg[BMAX];

      anum = hdr->confh;
      anum *= 256;
      anum += hdr->confl;

      strcpy(replymsg,olrdir);
      strcat(replymsg,"0_");
      ptr = replymsg;
      ptr += strlen(replymsg);
      ptr -= 2;
      *ptr += line;
      strcat(replymsg,"BLT-0.0");
      replychan = io_open(replymsg,3);

      olrlog = io_open(logfile,0);              /* open the logfile */
      if(olrlog < 0)
        olrlog = io_open(logfile,2);            /* open new */
      if(olrlog < 0)
        olrlog = io_open("ram1_olr_log",3);     /* open over */

      fs_pos(olrlog,0,2);                       /* goto file end */

      strncpy(subj,hdr->msgfrom,25);
      ptr = &subj[24];
      while(*ptr == ' ') ptr--;
      *(++ptr)='\0';
      sprintf(temp,"\n%02d/%02d/%02d, %02d:%02d Request from %s\n",
             now->tm_mday,1+(now->tm_mon),now->tm_year,
             now->tm_hour,now->tm_min,subj);
      io_sstrg(olrlog,-1,temp,strlen(temp));

      ptr = lbuff;
      for(f=atoi(hdr->numblocks); f>1; f--)     /* read in the msg */
      {
        ferr = io_fstrg(fchan, 100, ptr, 128);
        if((ferr > 0) && f < (4096/128))        /* max nr of blocks */
          ptr += ferr;
          sz += ferr;
      }
      strncpy(subj,hdr->subject,25);
      ptr = &subj[25];
      while(*ptr == ' ') ptr--;
      *(++ptr)='\0';
 /*
    check for simple msg
 */
      if(strnicmp(subj,"ADD",3)==0)
      {
        ptr = &subj[3];
        while(*ptr == ' ') ptr++;               /* point to optional value */
        c = *ptr;
        if((c == '+') || (c == '-'))
          ptr++;
        else
          c = ' ';
        val = atoi(ptr);
        sprintf(temp,"Add area %d, %c%d",anum,c,val);
        ok = areafix(anum,c,val,1);
        add_reply(temp,ok);
      }
      else if(strnicmp(subj,"RESET",5)==0)
      {
        ptr = &subj[5];
        while(*ptr == ' ') ptr++;               /* point to optional value */
        c = *ptr;
        if((c == '+') || (c == '-'))
          ptr++;
        else
          c = ' ';
        val = atoi(ptr);
        sprintf(temp,"Reset %d, %c%d",anum,c,val);
        ok = areafix(anum,c,val,3);
        add_reply(temp,ok);
      }
      else if(strnicmp(subj,"DROP",4)==0)
      {
        sprintf(temp,"Drop area %d",anum);
        ok = areafix(anum,'\0',0,2);
        add_reply(temp,ok);
      }
      else if(strnicmp(subj,"REQUEST",7)==0)
      {
        ptr = &subj[7];
        while(*ptr == ' ') ptr++;               /* point to filename */
        sprintf(temp,"Request %s",ptr);
        ok = attach(ptr);
        add_reply(temp,ok);
      }
 /*
    is it a "config" msg
 */
      else if(strnicmp(subj,"CONFIG",6)==0)
      {
        char *parm1, *parm2, *parm3;
        int nparms;

        io_sstrg(olrlog,-1,"Config\n",7);

        lptr = lbuff;
        for(f=0; f<sz; f++)
        {
          if(*lptr == '\xe3')   /* char 227 */
             *lptr = '\0';
          lptr++;
        }

        lptr = lbuff;
        while(1)
        {
          if(lptr >= (lbuff+sz))                /* any more to process? */
            break;
          ptr = lptr;                           /* start of line */
          while(*lptr != '\0') lptr++;          /* end of line */
          lptr++;                               /* start of next line */
          nparms = 0;                           /* split the line up */
          parm1=ptr;
          while(*parm1 == ' ') parm1++;
          if(*parm1 != '\0')
          {
            nparms++;
            parm2 = parm1;
            while(isgraph(*parm2)) parm2++;     /* end of parm1 */
            c = *parm2;
            *parm2 = '\0';                      /* terminate it */
            if(c != '\0')                       /* any more params */
            {
              parm2++;
              while(*parm2 == ' ') parm2++;
              if(*parm2 != '\0')
              {
                nparms++;
                parm3 = parm2;
                while(isgraph(*parm3)) parm3++;     /* end of parm2 */
                c = *parm3;
                *parm3 = '\0';                      /* terminate it */
                if(c != '\0')                       /* any more params */
                {
                  parm3++;
                  while(*parm3 == ' ') parm3++;    /* parm3 NOT terminated */
                  if (*parm3 != '\0')              /* as its only used in  */
                  {                                /* atoi which doesnt care */
                    nparms++;
                  }
                }
              }
            }
          }
          if(strnicmp(parm1,"ADD",3)==0)
          {
            c = *parm3;
            if((c == '+') || (c == '-'))
              parm3++;
            else
              c = ' ';
            anum = get_areanum(parm2);
            val = 0;
            sprintf(temp,"Add area %d",anum);
            if(nparms > 2)
            {
              ptr = temp;
              ptr += strlen(temp);
              val = atoi(parm3);
              sprintf(ptr,", from %c%d",c,val);
            }
            ok = areafix(anum,c,val,1);
          }
          else if(strnicmp(parm1,"DROP",4)==0)
          {
            anum = get_areanum(parm2);
            sprintf(temp,"Drop area %d",anum);
            ok = areafix(anum,'\0',0,2);
          }
          else if(strnicmp(parm1,"RESET",5)==0)
          {
            c = *parm3;
            if((c == '+') || (c == '-'))
              parm3++;
            else
              c = ' ';
            anum = get_areanum(parm2);
            val = 0;
            sprintf(temp,"Reset area %d",anum);
            if(nparms > 2)
            {
              ptr = temp;
              ptr += strlen(temp);
              val = atoi(parm3);
              sprintf(ptr," to %c%d",c,val);
            }
            ok = areafix(anum,c,val,3);
          }
          else if(strnicmp(parm1,"YOUR",4)==0)
          {
            c = *parm3;
            if((c == '+') || (c == '-'))
              parm3++;
            else
              c = ' ';
            anum = get_areanum(parm2);
            val = 0;
            sprintf(temp,"Your messages in area %d",anum);
            if(nparms > 2)
            {
              ptr = temp;
              ptr += strlen(temp);
              val = atoi(parm3);
              sprintf(ptr,", from %c%d",c,val);
            }
            ok = areafix(anum,c,val,4);
          }
          else if(strnicmp(parm1,"YA",2)==0)
          {
            c = *parm3;
            if((c == '+') || (c == '-'))
              parm3++;
            else
              c = ' ';
            anum = get_areanum(parm2);
            val = 0;
            sprintf(temp,"Messages to You and All in area %d",anum);
            if(nparms > 2)
            {
              ptr = temp;
              ptr += strlen(temp);
              val = atoi(parm3);
              sprintf(ptr,", from %c%d",c,val);
            }
            ok = areafix(anum,c,val,5);
          }
          else if(strnicmp(parm1,"REQUEST",7)==0)
          {
            sprintf(temp,"Request %s",parm2);
            ok = attach(parm2);
          }
          else if(strnicmp(parm1,"ARCHIVER",8)==0)
          {
            sprintf(temp,"setline %d oarc %c",line,*parm2);
            ptr=Request ("pbox", temp, strlen(temp), &anum);
            if (ptr != NULL) free(ptr);
            sprintf(temp,"Archiver %s",parm2);
            ok = 1;     /* no check for valid type! */
          }
          else if(strnicmp(parm1,"PROTOCOL",8)==0)
          {
            sprintf(temp,"setline %d opro %c",line,*parm2);
            ptr=Request ("pbox", temp, strlen(temp), &anum);
            if (ptr != NULL) free(ptr);
            sprintf(temp,"Protocol %s",parm2);
            ok = 1;
          }
          else if(strnicmp(parm1,"WELCOME",7)==0)
          {
            sprintf(temp,"Welcome %s",parm2);
            ok = setopt('W',parm2);
          }
          else if(strnicmp(parm1,"GOODBYE",7)==0)
          {
            sprintf(temp,"Goodbye %s",parm2);
            ok = setopt('G',parm2);
          }
          else if(strnicmp(parm1,"FILELIST",8)==0)
          {
            sprintf(temp,"Filelist %s",parm2);
            ok = setopt('F',parm2);
          }
          else if(strnicmp(parm1,"NEWLIST",7)==0)
          {
            sprintf(temp,"Newlist %s",parm2);
            ok = setopt('N',parm2);
          }
          else if(strnicmp(parm1,"BULLETINS",9)==0)
          {
            sprintf(temp,"Bulletins %s",parm2);
            ok = setopt('B',parm2);
          }
          else if(strnicmp(parm1,"HELP",4)==0)
          {
            sprintf(temp,"Help %s",parm2);
            ok = setopt('H',parm2);
          }
          else if(strnicmp(parm1,"INDEX",5)==0)
          {
            sprintf(temp,"Index %s",parm2);
            ok = setopt('I',parm2);
          }
          else if(strnicmp(parm1,"AREALIST",8)==0)
          {
            sprintf(temp,"Arealist %s",parm2);
            ok = setopt('A',parm2);
          }
          else if(strnicmp(parm1,"CONFLIST",8)==0)
          {
            sprintf(temp,"Conflist %s",parm2);
            ok = setopt('C',parm2);
          }
          else          /* unrecognised request */
          {
            sprintf(temp,"\"%s\"",parm1);
            ok = -1;
          }
          add_reply(temp, ok);
        }
      }
      io_close(olrlog);
      io_close(replychan);
}
/*--------------------------------------------------------------------------*/
int  main(int argc, char * argv[])
{
char *a, *t;
int   n;
char  temp[40];
struct   qdirect qd;            /* structure for reading directories  */
long  dirch;
short rlen;
int   ferr;                     /* file error return */
char  fname[40];                /* file to process */
char  lbuff[130];               /* buffer for lines */
char  bbsid[12];
QWKHEADER *header;

    if(argc == 2)               /* get line number if supplied */
      line = (char)atoi(argv[1]);
    else
      line = 0;

    if((n=UseSrvThg (CLNT, "pbox")) == 0)
    {
       loglevel = 0xffff;
       t = Request ("pbox","sysvar loglevel",-1,&rlen);
       if (t != NULL)
       {
         t[rlen] = '\0';
         if (*t != '-')
           loglevel = atoi(t);
         free(t);
       }
       sprintf(lbuff,"%s %s",_prog_name,_version);
       log(1,lbuff);

      sysvar("sysdir", sysdir, sizeof(sysdir)-1);
      if(sysdir[0] == '\0')
      {
        log(32,"-Error unable to access sysdir");
        FreeSrvThg (CLNT, "pbox");
        return(-1);
      }
      sysvar("inbound", inbound, sizeof(inbound)-1);
      if(inbound[0] == '\0')
      {
        log(32,"-Error unable to access inbound");
        FreeSrvThg (CLNT, "pbox");
        return(-1);
      }
      sysvar("storedir", store, sizeof(store)-1);
      sysvar("tempdir", tempdir, sizeof(tempdir)-1);
      sysvar("olrlog",logfile,BMAX-1);
      sysvar("olrdir",olrdir,BMAX-1);
      sysvar("userdir",userdir,BMAX-1);
      sysvar("unzip",unzip,BMAX-1);
      sysvar("unarc",unarc,BMAX-1);
      sysvar("unarj",unarj,BMAX-1);
      sysvar("unlzh",unlzh,BMAX-1);
      sysvar("unhar",unhar,BMAX-1);

      sysvar("mareas", mareas, sizeof(mareas)-1);
      flch = io_open( mareas, 1 );         /* mareas file is read only */
      if (flch > 0 )
         dbid = fsd_open(flch);

      if (flch < 0 || dbid < 0 )           /* exit if cant open mareas_dbs */
      {
        (void)strcpy(logmsg,"Bad mareas file");
        log(32,logmsg);
        return(-1);
      }
      qltime =  mt_rclck();             /* get ql time */
      rnum   =  qltime;
      qltime -= unixtoql;               /* in unix base (1961)  */
      now    =  localtime(&qltime);     /* fill time structure  */

/*------------------------------------
        unpack any reply bundles found
  ------------------------------------*/
     sprintf(temp,"%s*rep",inbound);     /* wildcard all replies in inbound */

     dirch = open_qdir(temp);
     while(read_qdir(dirch, temp, fname, &qd, 0))
     {
       (void)unpack(fname);
     }
     (void)io_close(dirch);
/*------------------------------------
        process any replies
  ------------------------------------*/
     sprintf(temp,"%s*msg",inbound);
     dirch = open_qdir(temp);
     while(read_qdir(dirch, temp, fname, &qd, 0))
     {
       fchan = io_open(fname,1);
       if(fchan < 0)
         return(fchan);
       sysvar("bbsid", bbsid, sizeof(bbsid)-1);
       ferr = io_fstrg(fchan, 100, lbuff, 128);  /* read header line */
       if(strnicmp(lbuff,bbsid,strlen(bbsid)) != 0)
         return(-15);                            /* make sure it's for us */
       open_pkt();
       while(1)                                  /* process the file */
       {
         ferr = io_fstrg(fchan, 100, lbuff, 128); /* message header */
         if (ferr < 0 )                      /* oops - probably eof */
           break;
         header = (QWKHEADER *)lbuff;
         if(strnicmp(header->msgto,"PBOX",4) == 0)
            process_request(header);
         else
            add_to_pkt(header);
       }
       io_close(fchan);
       io_delete(fname);                /* remove the packet once processed */
       close_pkt();
     }
     (void)io_close(dirch);
     (void)fsd_close(dbid);
     (void)io_close(flch);
      return(0);
    }
}
/*------------------------------- END ---------------------------------------*/
